@@ -114,6 +114,8 @@
 #include <netinet/in.h>
 #endif
 
+#include <string.h>
+
 #define NOT_IMPLEMENTED 0
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -194,9 +196,7 @@ enum
 
   PROP_RESEND_STREAMHEADER,
 
-  PROP_NUM_HANDLES,
-
-  PROP_LAST
+  PROP_NUM_HANDLES
 };
 
 GType
@@ -456,8 +456,7 @@ gst_multi_handle_sink_class_init (GstMultiHandleSinkClass * klass)
       G_STRUCT_OFFSET (GstMultiHandleSinkClass, clear), NULL, NULL,
       g_cclosure_marshal_generic, G_TYPE_NONE, 0);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Multi socket sink", "Sink/Network",
@@ -677,6 +676,10 @@ gst_multi_handle_sink_add_full (GstMultiHandleSink * sink,
   if (clink != NULL)
     goto duplicate;
 
+  /* We do not take ownership of @handle in this function, but we can't take a
+   * reference directly as we don't know the concrete type of the handle.
+   * GstMultiHandleSink relies on the derived class to take a reference for us
+   * in new_client: */
   mhclient = mhsinkclass->new_client (mhsink, handle, sync_method);
 
   /* we can add the handle now */
@@ -1138,13 +1141,9 @@ gst_multi_handle_sink_client_queue_buffer (GstMultiHandleSink * mhsink,
 static gboolean
 is_sync_frame (GstMultiHandleSink * sink, GstBuffer * buffer)
 {
-  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT)) {
+  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT))
     return FALSE;
-  } else if (!GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_HEADER)) {
-    return TRUE;
-  }
-
-  return FALSE;
+  return TRUE;
 }
 
 /* find the keyframe in the list of buffers starting the
@@ -1842,10 +1841,64 @@ restart:
   }
 }
 
+static gboolean
+buffer_is_in_caps (GstMultiHandleSink * sink, GstBuffer * buf)
+{
+  GstCaps *caps;
+  GstStructure *s;
+  const GValue *v;
+
+  caps = gst_pad_get_current_caps (GST_BASE_SINK_PAD (sink));
+  if (!caps)
+    return FALSE;
+  s = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_has_field (s, "streamheader")) {
+    gst_caps_unref (caps);
+    return FALSE;
+  }
+
+  v = gst_structure_get_value (s, "streamheader");
+  if (GST_VALUE_HOLDS_ARRAY (v)) {
+    guint n = gst_value_array_get_size (v);
+    guint i;
+    GstMapInfo map;
+
+    gst_buffer_map (buf, &map, GST_MAP_READ);
+
+    for (i = 0; i < n; i++) {
+      const GValue *v2 = gst_value_array_get_value (v, i);
+      GstBuffer *buf2;
+      GstMapInfo map2;
+
+      if (!GST_VALUE_HOLDS_BUFFER (v2))
+        continue;
+
+      buf2 = gst_value_get_buffer (v2);
+      if (buf == buf2) {
+        gst_caps_unref (caps);
+        return TRUE;
+      }
+      gst_buffer_map (buf2, &map2, GST_MAP_READ);
+      if (map.size == map2.size && memcmp (map.data, map2.data, map.size) == 0) {
+        gst_buffer_unmap (buf2, &map2);
+        gst_buffer_unmap (buf, &map);
+        gst_caps_unref (caps);
+        return TRUE;
+      }
+      gst_buffer_unmap (buf2, &map2);
+    }
+    gst_buffer_unmap (buf, &map);
+  }
+
+  gst_caps_unref (caps);
+
+  return FALSE;
+}
+
 static GstFlowReturn
 gst_multi_handle_sink_render (GstBaseSink * bsink, GstBuffer * buf)
 {
-  gboolean in_caps;
+  gboolean is_header, in_caps;
 #if 0
   GstCaps *bufcaps, *padcaps;
 #endif
@@ -1866,8 +1919,9 @@ gst_multi_handle_sink_render (GstBaseSink * bsink, GstBuffer * buf)
     goto no_caps;
 #endif
 
-  /* get IN_CAPS first, code below might mess with the flags */
-  in_caps = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_HEADER);
+  /* get HEADER first, code below might mess with the flags */
+  is_header = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_HEADER);
+  in_caps = is_header && buffer_is_in_caps (sink, buf);
 
 #if 0
   /* stamp the buffer with previous caps if no caps set */
@@ -1904,32 +1958,15 @@ gst_multi_handle_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
-  /* if we get IN_CAPS buffers, but the previous buffer was not IN_CAPS,
-   * it means we're getting new streamheader buffers, and we should clear
-   * the old ones */
-  if (in_caps && sink->previous_buffer_in_caps == FALSE) {
-    GST_DEBUG_OBJECT (sink,
-        "receiving new HEADER buffers, clearing old streamheader");
-    g_slist_foreach (sink->streamheader, (GFunc) gst_mini_object_unref, NULL);
-    g_slist_free (sink->streamheader);
-    sink->streamheader = NULL;
-  }
-
-  /* save the current in_caps */
-  sink->previous_buffer_in_caps = in_caps;
-
-  /* if the incoming buffer is marked as IN CAPS, then we assume for now
-   * it's a streamheader that needs to be sent to each new client, so we
-   * put it on our internal list of streamheader buffers.
-   * FIXME: we could check if the buffer's contents are in fact part of the
-   * current streamheader.
+  /* if the incoming buffer is a streamheader from the caps, then we assume for now
+   * it's a streamheader that needs to be sent to each new client.
    *
    * We don't send the buffer to the client, since streamheaders are sent
    * separately when necessary. */
   if (in_caps) {
-    GST_DEBUG_OBJECT (sink, "appending HEADER buffer with length %"
-        G_GSIZE_FORMAT " to streamheader", gst_buffer_get_size (buf));
-    sink->streamheader = g_slist_append (sink->streamheader, buf);
+    GST_DEBUG_OBJECT (sink, "ignoring HEADER buffer with length %"
+        G_GSIZE_FORMAT, gst_buffer_get_size (buf));
+    gst_buffer_unref (buf);
   } else {
     /* queue the buffer, this is a regular data buffer. */
     gst_multi_handle_sink_queue_buffer (sink, buf);
@@ -2107,7 +2144,6 @@ gst_multi_handle_sink_start (GstBaseSink * bsink)
   if (!mhsclass->start_pre (mhsink))
     return FALSE;
 
-  mhsink->streamheader = NULL;
   mhsink->bytes_to_serve = 0;
   mhsink->bytes_served = 0;
 
@@ -2152,12 +2188,6 @@ gst_multi_handle_sink_stop (GstBaseSink * bsink)
   /* free the clients */
   mhclass->clear (GST_MULTI_HANDLE_SINK (mhsink));
 
-  if (mhsink->streamheader) {
-    g_slist_foreach (mhsink->streamheader, (GFunc) gst_mini_object_unref, NULL);
-    g_slist_free (mhsink->streamheader);
-    mhsink->streamheader = NULL;
-  }
-
   if (mhclass->close)
     mhclass->close (mhsink);
 
@@ -2191,9 +2221,16 @@ gst_multi_handle_sink_change_state (GstElement * element,
   sink = GST_MULTI_HANDLE_SINK (element);
 
   /* we disallow changing the state from the streaming thread */
-  if (g_thread_self () == sink->thread)
-    return GST_STATE_CHANGE_FAILURE;
+  if (g_thread_self () == sink->thread) {
+    g_warning
+        ("\nTrying to change %s's state from its streaming thread would deadlock.\n"
+        "You cannot change the state of an element from its streaming\n"
+        "thread. Use g_idle_add() or post a GstMessage on the bus to\n"
+        "schedule the state change from the main thread.\n",
+        GST_ELEMENT_NAME (sink));
 
+    return GST_STATE_CHANGE_FAILURE;
+  }
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
